clear;
%version 092215 Halley's updated code using Histc 
% Delete processed files from directory before each run
load pgtabley.mat; % Load lookup table for sigma y
load pgtablez.mat; % Load lookup table for sigma z
files = dir( 'STR*.xls');
countfiles = numel(files);

run = 12041502;   % Analysis run MMDDYY(see notbook)
wslimit = 0.0;  % Set wind speed cut limit (m/s)
al = 60;       % Set wind angle cut limit (+/- deg, 180 indicates no filter
clp = 2.0;      % Set bin density cut limit (%)
bb1 = 0          % Set start deg value for angle bin (deg)
bb2 =10         % Set bin size (deg)
bb3 = 360       % Set end bin (see Ln 12)
degB = bb1:bb2:bb3; % Wd bins
n1 = 40;    % Remove sampling time deplay

% Additional information for files
% Distances from observation to source
dist = [40.45;59.83;35.18;97.44;56.99;87.76;87.76;98.4;98.4;102.96;102.96;102.96;82;57.00;57.00;81.00;97.80;65.40;41.00;92.90];
% Sensor heights
senh = [2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69;2.69];
% Source heights
sorh = [3.0;3.0;3.0;3.0;3.0;3.0;3.0;3.0;3.0;3.0;3.0;3.0;3.0;3.0;3.0;2.0;3.0;3.0;3.0;3.0];

% Constants used
rt1rp1 = 298/1013.25;
gasconst = 8.314510;
mw = 16.04;
grt1rp1 = rt1rp1*gasconst;
rt0rp0 = (gasconst*298)/101.325;
opt = (1e-06 * mw*1000)/rt0rp0;

% Listing of variables used in code
impnum = [];
disney0 = [];
coeffi = [];
coeffi1 = [];
rsq = [];
cut0 = [];
numabcknd = [];
pnum = [];
rt2rp2 = [];
methanemean = [];
binqa = [];
cangl1 = [];
countqa = [];
tempqa = [];
stdws3tqa = [];
minws3tqa = [];
maxws3tqa = [];
a1bkg1 = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Loop through files
%k1 = 6
for k1 = 1:countfiles
    % Import File
    jadu1 = xlsread(files(k1).name);
    time = double(jadu1(:,1));
    time = time*24;
    co2 = jadu1((1:end),2);     % c02 concentration (ppm)
    ch4 = jadu1((1:end),3);     % Ch4 concentration (ppm)
    lat = jadu1((1:end),4);     % GPS latitude
    long = jadu1((1:end),5);    % GPS longitude
    ws3 = jadu1((1:end),6);     % 3D sonic 3D wind speed (m/s)
    wd3 = jadu1((1:end),7);     % 3D wind direction (deg, arbitrary)
    ws2 = jadu1((1:end),8);     % 2D wind speed (m/s)
    wd2 = jadu1((1:end),9);     % 2D wind direction (deg, auto aligned north)
    temp = jadu1((1:end),10);   % Met station Temperature (deg C)
    pres = jadu1((1:end),12);   % Met Station Pressure (mbar)
    ws3z = jadu1((1:end),13);   % 3D w  (m/s)
    ws3x = jadu1((1:end),14);   % 3D u  (m/s)
    ws3y = jadu1((1:end),15);   % 3D v  (m/s)
    ws3t = jadu1((1:end),16);   % 3D t  (deg C)
    
    % Rewrite all arrays to remove time delay n1
    co2(1:n1) = [];
    ch4(1:n1) = [];
    c1 = length(co2);
    time(c1+1:end) = [];
    ws3(c1+1:end) = [];
    wd3(c1+1:end) = [];
    ws2(c1+1:end) = [];
    wd2(c1+1:end) = [];
    temp(c1+1:end) = [];
    pres(c1+1:end) = [];
    ws3z(c1+1:end) = [];
    ws3x(c1+1:end) = [];
    ws3y(c1+1:end) = [];
    ws3t(c1+1:end) = [];
    lat(c1+1:end) = [];
    long(c1+1:end) = [];
    
    % Rotate 3D sonic coordinate system to streamlined set (-180 deg)
    % First rotation (set <x> = 0)
    RA = pi + (atan2(-1*(mean(ws3x)),-1*(mean(ws3y))));
    nws3y = ws3y*cos(RA) + ws3x*sin(RA);
    nws3x = (-1)*ws3y*sin(RA) + ws3x*cos(RA);
    % Second rotation (set <z> = 0)
    RB = pi + (atan2(-1*(mean(ws3z)),-1*(mean(nws3y))));
    nnws3y = nws3y*cos(RB) + ws3z*sin(RB);
    nws3z = (-1)*nws3y*sin(RB) + ws3z*cos(RB);
    % Assigned streamlined coordates
    ws3y = nnws3y;
    ws3x = nws3x;
    ws3z = nws3z;
    % Calculate and asign new 3D sonic 2D wind direction
    wd3 = 180 + (atan2(-1*(ws3x),-1*(ws3y))*57.29578);
    % Calculate and asign new 3D sonic 2D wind speed
    ws3 = (ws3y.*ws3y + ws3x.*ws3x).^0.5;
    wd3(wd3<0) = 0; 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Wind angle filter
    % Calculate mean ch4 conc by wd bin;
    % Threshold for number of measurements per bin (2%)
    cut = clp/100 * length(wd3);
    %[countM, edges, wdbin] = histcounts(wd3, degB);
    [countM2, wdbin] = histc(wd3, degB);
    countM = countM2(1:(size(degB,2)-1)).';
    resM = accumarray(wdbin, ch4, [], @(x) mean(x));
    % Set bin-values below threshold to 0;
    resM(find(countM < cut)) = 0;
    % Find maximum;
    [C,binx] = max(resM);
    binxx = (bb2-bb1)*(binx) - bb2/2;
    % If maximum near 0 or 360 edges, rotate wd3 by 180
    if (binxx < 90 | binxx > 270);
        wd3(find(wd3 > 180)) = wd3(find(wd3 > 180)) - 360;
        wd3 = wd3 + 180;
        if (binxx > 270);
            binxx = binxx - 180;
        else;
            binxx = binxx+180;
        end;
    end;
    
    % Filter file based on wind criteria
    jadu2 = [time co2 ch4 ws3 wd3 ws2 wd2 temp pres ws3z lat long ws3x ws3y ws3t];
    % Windspeed filter
    jadu2(find(ws3 < wslimit),:) = [];
    jadu2(find(wd3 < (binxx - al) | wd3 > (binxx + al)),:) = [];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    a1 = jadu2;
    ws = a1(:,4);               % 3D soinic 2D wind speed (m/s)
    ws2 = a1(:,6);              % Met. Station 2D wind wpeed (m/s)
    temp = a1(:,8)+273.15;      % Met. Station temp (deg K)
    pres = a1(:,9);             % Met. Station atmospheric pressure (mbar)
    a1a = a1(:,5);              % 3D wind direction
    a1b = a1(:,3);              % Methane concentration
    a1c = a1(:,2);              % CO2 concentration
    ws3z = a1(:,10);            % 3D w
    ws3x = a1(:,13);            % 3D u
    ws3y = a1(:,14);            % 3D v
    ws3t = a1(:,15);            % 3D t
    maxmet = max(a1b);          % Maximum methane concentration
    minmet = min(a1b);          % Minimum methane concentration
    mmet = mean(a1b);           % Mean methane concentration
    mws = mean(ws);             % Mean 3D sonic wind speed (m/s)
    mws2 = mean(ws2);           % Mean met. Station 2D wind speed (m/s)
    mtemp = mean(temp);         % Mean met station temp. (K)
    mpres = mean(pres);         % Mean met. station atmospheric pressure (mbar)
    m3Dt = mean(ws3t)+273.15;   % Mean 3D sonic temp. (K)
    mco2 = mean(a1c);           % Mean CO2 concentration (ppm)
    stdco2 = std(a1c);          % Std dev in CO2 (ppm)
    stdws3t = std(ws3t);        % Std dev in 3D sonic t (C)
    minws3t = min(ws3t);        % Minimum in 3D sonic t (C)
    maxws3t = max(ws3t);        % Maximum in 3D sonic t (C)
    lat = a1(:,11);
    long = a1(:,12);
    lat1 = mean(lat(:,1));
    long1 = mean(long(:,1));
    latstd = std(lat);
    longstd = std(long);
    
    disney = files(k1).name;
    disney1 = isletter(disney);
    disney2 = find(disney1 == 0);
    disney3 = disney(disney2);
    disney4 = str2num(disney3(2:8));
    disney5 = str2num(disney3(10:end));
    disney0 = [disney0; disney4 disney5];
    
    % Taking the average of the lowest methane concentration values
    og = length(a1b);
    oga = round(og*0.05);
    lowval = sort(a1b);
    lowmet = mean(lowval(1:oga)); %lowest 5% values
    methanemean = [methanemean;lowmet];
    mmmet = mmet - lowmet;
    lowmetstd = std(lowval(1:oga));
    hival = sort(a1b,'descend');
    himet = mean(hival(1:oga));
    himetstd = std(hival(1:oga));
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %calculating raw mean products for use in Windtrax
    my = mean(ws3x);
    mx = mean(ws3y);
    mz = mean(ws3z);
    mt = mean(ws3t);
    y2 = ws3x.*ws3x;
    my2 = mean(y2);
    x2 = ws3y.*ws3y;
    mx2 = mean(x2);
    z2 = ws3z.*ws3z;
    mz2 = mean(z2);
    t2 = ws3t.*ws3t;
    mt2 = mean(t2);
    yx = ws3x.*ws3y;
    myx =  mean(yx);
    yz = ws3x.*ws3z;
    myz =  mean(yz);
    yt = ws3x.*ws3t;
    myt =  mean(yt);
    xz = ws3y.*ws3z;
    mxz =  mean(xz);
    xt = ws3y.*ws3t;
    mxt =  mean(xt);
    zt = ws3z.*ws3t;
    mzt =  mean(zt);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%calculating wind parameters
    
    %turbulent intensity
    sws3z = std(ws3z);
    turbuint = sws3z/mws;
    
    % Average and the standard deviation of wind 3D sonic azmuth wind direction
    
    wd3mean = 180 + (atan2(-1*(my),-1*(mx))*57.29578);
    stddev3 = std(a1a);
    
    % Average and the standard deviation of wind direction for 2D met station
    wd2rad = a1(:,7) * pi/180;
    wd2y = sin(wd2rad);
    wd2x = cos(wd2rad);
    wd2num = numel(wd2rad);
    wd2xmean = mean(wd2x);
    wd2ymean = mean(wd2y);
    
    if wd2xmean > 0 && wd2ymean > 0
        wd2mean = atan(wd2ymean/wd2xmean);
        wd2mean = wd2mean * (180/pi);
        
        ep1 = wd2xmean.^2 + wd2ymean.^2;
        ep1 = sqrt(1-ep1);
        stddev1 = asin(ep1);
        stddev2 = 1+(1.1547 -1)*ep1.^3;
        stddev = stddev1 * stddev2;
        stddev = stddev * (180/pi);
        
    elseif wd2xmean > 0 && wd2ymean < 0
        wd2mean = atan(wd2ymean/wd2xmean);
        wd2mean = wd2mean * (180/pi);
        wd2mean = wd2mean + 360;
        
        ep1 = wd2xmean.^2 + wd2ymean.^2;
        ep1 = sqrt(1-ep1);
        stddev1 = asin(ep1);
        stddev2 = 1+(1.1547 -1)*ep1.^3;
        stddev = stddev1 * stddev2;
        stddev = stddev * (180/pi);
        
    elseif wd2xmean < 0
        wd2mean = atan(wd2ymean/wd2xmean);
        wd2mean = wd2mean * (180/pi);
        wd2mean = wd2mean + 180;
        
        ep1 = wd2xmean.^2 + wd2ymean.^2;
        ep1 = sqrt(1-ep1);
        stddev1 = asin(ep1);
        stddev2 = 1+(1.1547 -1)*ep1.^3;
        stddev = stddev1 * stddev2;
        stddev = stddev * (180/pi);
        
    end
    
    %%%%%%%%%%%%%%%%% Seciton estimates sigmay and sigmaz estimate based on stability class estimate
    
    %%% Wind Speed PG Indicator (3D)
    if mws > 0 && mws < 1.700;
        pgwsi = 1;
    elseif mws >= 1.700 && mws < 2.400;
        pgwsi = 2;
    elseif mws >= 2.400 && mws < 3.200;
        pgwsi = 3;
    elseif mws >= 3.200 && mws < 4.100;
        pgwsi = 4;
    elseif mws >= 4.100 && mws < 5.100;
        pgwsi = 5;
    elseif mws >= 5.100 && mws < 6.200;
        pgwsi = 6;
    elseif mws >= 6.200;
        pgwsi = 7;
    end
    
    %%% Wind Speed PG Indicator (2D)
    if mws > 0 && mws < 1.700;
        pgws2i = 1;
    elseif mws >= 1.700 && mws < 2.400;
        pgws2i = 2;
    elseif mws >= 2.400 && mws < 3.200;
        pgws2i = 3;
    elseif mws >= 3.200 && mws < 4.100;
        pgws2i = 4;
    elseif mws >= 4.100 && mws < 5.100;
        pgws2i = 5;
    elseif mws >= 5.100 && mws < 6.200;
        pgws2i = 6;
    elseif mws >= 6.200;
        pgws2i = 7;
    end
    
    %%% Stdev Wind Direction PG Indicator (3D)
    if stddev3 > 27.50;
        pgstddev3i = 1.00;
    elseif stddev3 <= 27.50 && stddev3 > 23.50;
        pgstddev3i = 2.00;
    elseif stddev3 <= 23.50 && stddev3 > 19.50;
        pgstddev3i = 3.00;
    elseif stddev3 <= 19.50 && stddev3 > 15.50;
        pgstddev3i = 4.00;
    elseif stddev3 <= 15.50 & stddev3 > 11.50;
        pgstddev3i = 5.00;
    elseif stddev3 <= 11.50 & stddev3 > 7.50;
        pgstddev3i = 6.00;
    elseif stddev3 <= 7.50
        pgstddev3i = 7.00;
    end
    
    %%% Stdev Wind Direction PG Indicator (2D)
    if stddev > 27.50;
        pgstddevi = 1.00;
    elseif stddev <= 27.50 && stddev > 23.50;
        pgstddevi = 2.00;
    elseif stddev <= 23.50 && stddev > 19.50;
        pgstddevi = 3.00;
    elseif stddev <= 19.50 && stddev > 15.50;
        pgstddevi = 4.00;
    elseif stddev <= 15.50 && stddev > 11.50;
        pgstddevi = 5.00;
    elseif stddev <= 11.50 && stddev > 7.50;
        pgstddevi = 6.00;
    elseif stddev <= 7.50
        pgstddevi = 7.00;
    end
    
    %%% Turbulent Intensity PG Indicator (3D)
    if turbuint > 0.205;
        pgturbi = 1.00;
    elseif turbuint <= 0.205 && turbuint > 0.180;
        pgturbi = 2.00;
    elseif turbuint <= 0.180 && turbuint > 0.155;
        pgturbi = 3.00;
    elseif turbuint <= 0.155 && turbuint > 0.130;
        pgturbi = 4.00;
    elseif turbuint <= 0.130 && turbuint > 0.105;
        pgturbi = 5.00;
    elseif turbuint <= 0.105 && turbuint > 0.080;
        pgturbi = 6.00;
    elseif turbuint <= 0.080;
        pgturbi = 7.00;
    end
    
    %%%%this line calculates average pg indicator
    
    pgi = [pgstddevi + pgturbi]/2;
    
    %%% PGtable select
    if pgi < 1.500;
        pgt = 1.00;
    elseif pgi >= 1.500 && pgi < 2.500;
        pgt = 2.00;
    elseif pgi >= 2.500 && pgi < 3.500;
        pgt = 3.00;
    elseif pgi >= 3.500 && pgi < 4.500;
        pgt = 4.00;
    elseif pgi >= 4.500 && pgi < 5.500;
        pgt = 5.00;
    elseif pgi >= 5.500 && pgi < 6.500;
        pgt = 6.00;
    elseif pgi >= 6.500
        pgt = 7.00;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    d = dist(k1);
    dd = round(d);
    pgty = pgtabley(dd,pgt);
    pgtz = pgtablez(dd,pgt);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%end of pg indicator section
    % Saving values
    impnum1= [mpres my mx mz mt my2 mx2 mz2 mt2 myx myz myt mxz mxt mzt lat1 latstd long1 longstd mco2 stdco2 oga maxmet himet himetstd minmet lowmet lowmetstd mmet mmmet mws mtemp wd2mean wd3mean stddev stddev3 turbuint pgws2i pgwsi pgstddevi pgstddev3i pgturbi pgi pgt pgty pgtz];
    impnum = [impnum; impnum1];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Binning measured concetration in in given degree increments
    % Subtracting off the background which is the avg of the lowest 5% values
    a1b = a1b - lowmet;
    
    % Bin background subtracted CH4 by wd
    %[count, edges, wdbin] = histcounts(a1a, degB);
    [count2, wdbin] = histc(a1a, degB);
    count = count2(1:(size(degB,2)-1)).';
    resN = accumarray(wdbin, a1b, [], @(x) mean(x));
    % Set bin-values below threshold to 0;
    resN(find(count < cut)) = 0;
    % Find maximum;
    [C,bqa] = max(resN);
    cangl = (bb2-bb1)*(bqa) - bb2/2;
    
    % QA indicators
    cqa = max(count);
    tqa = mtemp/m3Dt;
    binqa =[binqa;bqa];
    cangl1 =[cangl1;cangl];
    countqa =[countqa;cqa];
    tempqa =[tempqa;tqa];
    stdws3tqa = [stdws3tqa;stdws3t];
    minws3tqa = [minws3tqa;minws3t];
    maxws3tqa = [maxws3tqa;maxws3t];
    
    % Centers data so it is located at 180 degree
    degcent = 180;     % center of the binning
    degmax = 360;      % maximum of the binning
    degmin = 0;        % minimum of the binning
    resNmax = find(resN == max(resN));
    resNbtwn = find(resN >= min(resN) & resN <= max(resN));
    x = (degcent - degB(resNmax));
    degc = degB + x;
    for i1 = 1:length(degc)
        if degc(i1) > degmax
            degc(i1) = degc(i1)-degmax;
        elseif degc(i1) < degmax && degc(i1) >= degmin
            degc(i1) = degc(i1);
        elseif degc(i1) < degmin
            degc(i1) = degc(i1) + degmax;
        end
    end
    
    degc = degc(2:end)-bb2/2;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Fitting routine
    resNT = resN';
    degT = degc';
    fw = ezfit(degT, resNT ,'gauss');
    
    % Plot methane vs wind direction
    figure()
    plot(degc,resN,'b.','linewidth',0.5)
    set(gca,'fontsize',13);
    title('Average Methane Concentration vs. Wind Direction');
    ylabel('Methane Concetration (ppm)');
    xlabel ('Wind Direction (deg.)');
    grid on
    box on
    showfit(fw,'fitcolor','black','maxlengtheq',0);
    print('-djpeg','-r100',strcat(num2str(disney4),'_',num2str(disney5)));

    
    % Saving parameters to Summary processed files
    rt3rp3 = mpres/mtemp;
    %rt3rp3 = mpres/m3Dt;
    rt2rp2 = [rt2rp2;rt3rp3];
    coeffi = [coeffi;fw];
    disp(coeffi(k1).m(1:2))
    coeffi1 = [coeffi1; coeffi(k1).m(1:2)];
    rsq = [rsq; coeffi(k1).r];
    a1 = coeffi1(k1,1);
    a1bkg = lowmet + a1;
    a1bkg1 = [a1bkg1,a1bkg];
    
end

close all

% Converting to distance-based coordinates
sigmadeg = coeffi1(:,2);
sigmar = sigmadeg*(pi/180);
sigmadist = 0;
sigmadist = [];
for k3 = 1:length(dist)
    sigmadist1 = tan(sigmar(k3)/2)*2*dist(k3);
    sigmadist = [sigmadist;sigmadist1];
end

format short
mconct = coeffi1(:,1); % normalized methane concentration
mconctg = [];

% Calculating the peak methane concentration in g/m^3
for k4 = 1:length(mconct)
    mconct1 = mconct(k4) *opt* rt1rp1*rt2rp2(k4);
    mconctg = [mconctg;mconct1];
end

a1bkg2 = a1bkg1.';
impval = [disney0 dist senh sorh a1bkg2 methanemean impnum cangl1 ...
    bb2*ones(countfiles,1) wslimit*ones(countfiles,1)  ...
    al*ones(countfiles,1) clp*ones(countfiles,1) coeffi1 ...
    mconctg sigmadist rsq binqa countqa tempqa stdws3tqa ...
    minws3tqa maxws3tqa];

% Saving the calculated parameters in the excel sheets
dlmwrite('Summary_ProcessedWT2.csv', impval, 'precision', 10);



